<?php
/*!
 * C-Rent Web
 *
 * Copyright 2012, CX9 Systems GmbH
 * Authors: Christoph Weitkamp - c.weitkamp@cx9.de
 *
 * http://www.cx9.de
 */
// Class definition
class DBException extends Exception
{
    private $host;
    private $username;
    private $password;
    private $dbname;

    /*
     * constructor
     */
    public function __construct($message, $code = 0, $host = "", $username = "", $password = "", $dbname = "")
    {
        // set host
        $this->host = $host;
        // set user
        $this->username = $username;
        // set password
        $this->password = $password;
        // set database name
        $this->dbname = $dbname;

        // call constructor of parent class
        parent::__construct($message, $code);
    }

    /*
     * function setDBConfiguration()
     */
    public function setDBConfiguration($host, $username, $password, $dbname)
    {
        // set host
        $this->host = $host;
        // set user
        $this->username = $username;
        // set password
        $this->password = $password;
        // set database name
        $this->dbname = $dbname;
    }

    /*
     * function getDBConfiguration()
     */
    public function getDBConfiguration()
    {
        return "(" . $this->host . "," . $this->username . ",****," . $this->dbname . ")";
    }
}
// Class definition
class SQLException extends Exception
{
    //
    private $sql;

    /*
     * constructor
     */
    public function __construct($message, $code = 0, $sql = "")
    {
        // set sql statement
        $this->sql = $sql;

        // call constructor of parent class
        parent::__construct($message, $code);
    }

    /*
     * function setSQL()
     */
    public function setSQL($sql = "")
    {
        $this->sql = $sql;
    }

    /*
     * function setSQL()
     */
    public function getSQL()
    {
        return $this->sql;
    }
}
// Class definition
abstract class DBConnection
{
    protected $host;
    protected $username;
    protected $password;
    protected $dbname;

    /*
     * constructor
     */
    public function __construct($host, $username, $password, $dbname)
    {
        // set db host configuration
        $this->host = $host;
        // set db user configuration
        $this->username = $username;
        // generate password if not given in configuration file
        if (isset($password) && (strlen($password) > 0)) {
            $this->password = $password;
        } else {
            $this->password = DBConnection::get_password($username);
        }
        // set db configuration
        $this->dbname = $dbname;

        // connect to db
        if (PERSISTENT) {
            $this->connect(true);
        } else {
            $this->connect();
        }
    }

    /*
     * destructor
     */
    public function __destruct()
    {
        // disconnect from db
        $this->disconnect();
    }

    /*
     * function connect()
     * has to be defined in inheriting class
     */
    abstract public function connect($persistent = false);

    /*
     * function disconnect()
     * has to be defined in inheriting class
     */
    abstract public function disconnect();

    /*
     * function query()
     * has to be defined in inheriting class
     */
    abstract public function query($sql);

    /*
     * function fetch_row()
     * has to be defined in inheriting class
     */
    abstract public function fetch_row();

    /*
     * function fetch_array()
     * has to be defined in inheriting class
     */
    abstract public function fetch_array();

    /*
     * function fetch_assoc()
     * has to be defined in inheriting class
     */
    abstract public function fetch_assoc();

    /*
     * function free_result()
     */
    abstract public function free_result();

    /*
     * function real_escape_string()
     * has to be defined in inheriting class
     */
    public function real_escape_string($s)
    {
        return $s;
    }

    /*
     * function get_password()
     * generates a secret password from the given string
     */
    public static function get_password($s)
    {
        $replace = array(128 => "?", 130 => "?", 131 => "?", 132 => "?", 133 => "?", 134 => "?", 135 => "?", 136 => "?", 137 => "?",
            138 => "?", 139 => "?", 140 => "?", 142 => "?", 145 => "?", 146 => "?", 147 => "?", 148 => "?", 149 => "?",
            150 => "?", 151 => "?", 152 => "?", 153 => "?", 154 => "?", 155 => "?", 156 => "?", 158 => "?", 159 => "?",
            256 => "A", 257 => "a", 258 => "A", 259 => "a", 260 => "A", 261 => "a", 262 => "C", 263 => "c", 264 => "C",
            265 => "c", 266 => "C", 267 => "c", 268 => "C", 269 => "c", 270 => "D", 271 => "d", 272 => "Ð", 273 => "d",
            274 => "E", 275 => "e", 276 => "E", 277 => "e", 278 => "E", 279 => "e", 280 => "E", 281 => "e", 282 => "E",
            283 => "e", 284 => "G", 285 => "g", 286 => "G", 287 => "g", 288 => "G", 289 => "g", 290 => "G", 291 => "g",
            292 => "H", 293 => "h", 294 => "H", 295 => "h", 296 => "I", 297 => "i", 298 => "I", 299 => "i", 300 => "I",
            301 => "i", 302 => "I", 303 => "i", 304 => "I", 305 => "i", 306 => "?", 307 => "?", 308 => "J", 309 => "j",
            310 => "K", 311 => "k", 312 => "?", 313 => "L", 314 => "l", 315 => "L", 316 => "l", 317 => "L", 318 => "l",
            319 => "?", 320 => "?", 321 => "L", 322 => "l", 323 => "N", 324 => "n", 325 => "N", 326 => "n", 327 => "N",
            328 => "n", 329 => "?", 330 => "?", 331 => "?", 332 => "O", 333 => "o", 334 => "O", 335 => "o", 336 => "O",
            337 => "o", 340 => "R", 341 => "r", 342 => "R", 343 => "r", 344 => "R", 345 => "r", 346 => "S", 347 => "s",
            348 => "S", 349 => "s", 350 => "S", 351 => "s", 354 => "T", 355 => "t", 356 => "T", 357 => "t", 358 => "T",
            359 => "t", 360 => "U", 361 => "u", 362 => "U", 363 => "u", 364 => "U", 365 => "u", 366 => "U", 367 => "u",
            368 => "U", 369 => "u", 370 => "U", 371 => "u", 372 => "W", 373 => "w", 374 => "Y", 375 => "y", 377 => "Z",
            378 => "z", 379 => "Z", 380 => "z", 383 => "?", 384 => "b", 385 => "?", 386 => "?", 387 => "?", 388 => "?",
            389 => "?", 390 => "?", 391 => "?", 392 => "?", 393 => "Ð", 394 => "?", 395 => "?", 396 => "?", 397 => "?",
            398 => "?", 399 => "?", 400 => "?", 401 => "ƒ", 403 => "?", 404 => "?", 405 => "?", 406 => "?", 407 => "I",
            408 => "?", 409 => "?", 410 => "l", 411 => "?", 412 => "?", 413 => "?", 414 => "?", 415 => "O", 416 => "O",
            417 => "o", 418 => "?", 419 => "?", 420 => "?", 421 => "?", 422 => "?", 423 => "?", 424 => "?", 425 => "?",
            426 => "?", 427 => "t", 428 => "?", 429 => "?", 430 => "T", 431 => "U", 432 => "u", 433 => "?", 434 => "?",
            435 => "?", 436 => "?", 437 => "?", 438 => "z", 439 => "?", 440 => "?", 441 => "?", 442 => "?", 443 => "?",
            444 => "?", 445 => "?", 446 => "?", 447 => "?", 448 => "|", 449 => "?", 450 => "?", 451 => "!", 452 => "?",
            453 => "?", 454 => "?", 455 => "?", 456 => "?", 457 => "?", 458 => "?", 459 => "?", 460 => "?", 461 => "A",
            462 => "a", 463 => "I", 464 => "i", 465 => "O", 466 => "o", 467 => "U", 468 => "u", 469 => "U", 470 => "u",
            471 => "U", 472 => "u", 473 => "U", 474 => "u", 475 => "U", 476 => "u", 477 => "?", 478 => "A", 479 => "a",
            480 => "?", 481 => "?", 482 => "?", 483 => "?", 484 => "G", 485 => "g", 486 => "G", 487 => "g", 488 => "K",
            489 => "k", 490 => "O", 491 => "o", 492 => "O", 493 => "o", 494 => "?", 495 => "?", 496 => "j", 497 => "?",
            498 => "?", 499 => "?", 500 => "?", 501 => "?", 502 => "?", 503 => "?", 504 => "?", 505 => "?", 506 => "?",
            507 => "?", 508 => "?", 509 => "?", 510 => "?", 511 => "?", 512 => "?", 513 => "?", 514 => "?", 515 => "?",
            516 => "?", 517 => "?", 518 => "?", 519 => "?", 520 => "?", 521 => "?", 522 => "?", 523 => "?", 524 => "?",
            525 => "?", 526 => "?", 527 => "?", 528 => "?", 529 => "?", 530 => "?", 531 => "?", 532 => "?", 533 => "?",
            534 => "?", 535 => "?", 536 => "?", 537 => "?", 538 => "?", 539 => "?", 540 => "?", 541 => "?", 542 => "?",
            543 => "?", 544 => "?", 545 => "?", 546 => "?", 547 => "?", 548 => "?", 549 => "?", 550 => "?", 551 => "?",
            552 => "?", 553 => "?", 554 => "?", 555 => "?", 556 => "?", 557 => "?", 558 => "?", 559 => "?", 560 => "?",
            561 => "?", 562 => "?", 563 => "?", 564 => "?", 565 => "?", 566 => "?", 567 => "?", 568 => "?", 569 => "?",
            570 => "?", 571 => "?", 572 => "?", 573 => "?", 574 => "?", 575 => "?", 576 => "?", 577 => "?", 578 => "?",
            579 => "?", 580 => "?", 581 => "?", 582 => "?", 583 => "?", 584 => "?", 585 => "?", 586 => "?", 587 => "?",
            588 => "?", 589 => "?", 590 => "?", 591 => "?", 592 => "?", 593 => "?", 594 => "?", 595 => "?", 596 => "?",
            597 => "?", 598 => "?", 599 => "?", 600 => "?", 601 => "?", 602 => "?", 603 => "?", 604 => "?", 605 => "?",
            606 => "?", 607 => "?", 608 => "?", 609 => "g", 610 => "?", 611 => "?", 612 => "?", 613 => "?", 614 => "?",
            615 => "?", 616 => "?", 617 => "?", 618 => "?", 619 => "?", 620 => "?", 621 => "?", 622 => "?", 623 => "?",
            624 => "?", 625 => "?", 626 => "?", 627 => "?", 628 => "?", 629 => "?", 630 => "?", 631 => "?", 632 => "?",
            633 => "?", 634 => "?", 635 => "?", 636 => "?", 637 => "?", 638 => "?", 639 => "?", 640 => "?", 641 => "?",
            642 => "?", 643 => "?", 644 => "?", 645 => "?", 646 => "?", 647 => "?", 648 => "?", 649 => "?", 650 => "?",
            651 => "?", 652 => "?", 653 => "?", 654 => "?", 655 => "?", 656 => "?", 657 => "?", 658 => "?", 659 => "?",
            660 => "?", 661 => "?", 662 => "?", 663 => "?", 664 => "?", 665 => "?", 666 => "?", 667 => "?", 668 => "?",
            669 => "?", 670 => "?", 671 => "?", 672 => "?", 673 => "?", 674 => "?", 675 => "?", 676 => "?", 677 => "?",
            678 => "?", 679 => "?", 680 => "?", 681 => "?", 682 => "?", 683 => "?", 684 => "?", 685 => "?", 686 => "?",
            687 => "?", 688 => "?", 689 => "?", 690 => "?", 691 => "?", 692 => "?", 693 => "?", 694 => "?", 695 => "?",
            696 => "?", 697 => "'", 698 => "\"", 699 => "?", 700 => "'", 701 => "?", 702 => "?", 703 => "?", 704 => "?",
            705 => "?", 706 => "?", 707 => "?", 708 => "^", 709 => "?", 711 => "?", 712 => "'", 713 => "¯", 714 => "´",
            715 => "`", 716 => "?", 717 => "_", 718 => "?", 719 => "?", 720 => "?", 721 => "?", 722 => "?", 723 => "?",
            724 => "?", 725 => "?", 726 => "?", 727 => "?", 728 => "?", 729 => "?", 730 => "°", 731 => "?", 733 => "?",
            734 => "?", 735 => "?", 736 => "?", 737 => "?", 738 => "?", 739 => "?", 740 => "?", 741 => "?", 742 => "?",
            743 => "?", 744 => "?", 745 => "?", 746 => "?", 747 => "?", 748 => "?", 749 => "?", 750 => "?", 751 => "?",
            752 => "?", 753 => "?", 754 => "?", 755 => "?", 756 => "?", 757 => "?", 758 => "?", 759 => "?", 760 => "?",
            761 => "?", 762 => "?", 763 => "?", 764 => "?", 765 => "?", 766 => "?", 767 => "?", 768 => "`", 769 => "´",
            770 => "^", 771 => "~", 772 => "¯", 773 => "¯", 774 => "?", 775 => "?", 776 => "¨", 777 => "?", 778 => "°",
            779 => "?", 780 => "?", 781 => "?", 782 => "\"", 783 => "?", 784 => "?", 785 => "?", 786 => "?", 787 => "?",
            788 => "?", 789 => "?", 790 => "?", 791 => "?", 792 => "?", 793 => "?", 794 => "?", 795 => "?", 796 => "?",
            797 => "?", 798 => "?", 799 => "?", 800 => "?", 801 => "?", 802 => "?", 803 => "?", 804 => "?", 805 => "?",
            806 => "?", 807 => "¸", 808 => "?", 809 => "?", 810 => "?", 811 => "?", 812 => "?", 813 => "?", 814 => "?",
            815 => "?", 816 => "?", 817 => "_", 818 => "_", 819 => "?", 820 => "?", 821 => "?", 822 => "?", 823 => "?",
            824 => "?", 825 => "?", 826 => "?", 827 => "?", 828 => "?", 829 => "?", 830 => "?", 831 => "?", 832 => "?",
            833 => "?", 834 => "?", 835 => "?", 836 => "?", 837 => "?", 838 => "?", 839 => "?", 840 => "?", 841 => "?",
            842 => "?", 843 => "?", 844 => "?", 845 => "?", 846 => "?", 847 => "?", 848 => "?", 849 => "?", 850 => "?",
            851 => "?", 852 => "?", 853 => "?", 854 => "?", 855 => "?", 856 => "?", 857 => "?", 858 => "?", 859 => "?",
            860 => "?", 861 => "?", 862 => "?", 863 => "?", 864 => "?", 865 => "?", 866 => "?", 867 => "?", 868 => "?",
            869 => "?", 870 => "?", 871 => "?", 872 => "?", 873 => "?", 874 => "?", 875 => "?", 876 => "?", 877 => "?",
            878 => "?", 879 => "?", 880 => "?", 881 => "?", 882 => "?", 883 => "?", 884 => "?", 885 => "?", 886 => "?",
            887 => "?", 888 => "?", 889 => "?", 890 => "?", 891 => "?", 892 => "?", 893 => "?", 894 => ";", 895 => "?",
            896 => "?", 897 => "?", 898 => "?", 899 => "?", 900 => "?", 901 => "?", 902 => "?", 903 => "?", 904 => "?",
            905 => "?", 906 => "?", 907 => "?", 908 => "?", 909 => "?", 910 => "?", 911 => "?", 912 => "?", 913 => "?",
            914 => "?", 915 => "G", 916 => "?", 917 => "?", 918 => "?", 919 => "?", 920 => "T", 921 => "?", 922 => "?",
            923 => "?", 924 => "?", 925 => "?", 926 => "?", 927 => "?", 928 => "?", 929 => "?", 930 => "?", 931 => "S",
            932 => "?", 933 => "?", 934 => "F", 935 => "?", 936 => "?", 937 => "O", 938 => "?", 939 => "?", 940 => "?",
            941 => "?", 942 => "?", 943 => "?", 944 => "?", 945 => "a", 946 => "ß", 947 => "?", 948 => "d", 949 => "e",
            950 => "?", 951 => "?", 952 => "?", 953 => "?", 954 => "?", 955 => "?", 956 => "µ", 957 => "?", 958 => "?",
            959 => "?", 960 => "p", 961 => "?", 962 => "?", 963 => "s", 964 => "t", 965 => "?", 966 => "f", 967 => "?",
            968 => "?", 969 => "?", 970 => "?", 971 => "?", 972 => "?", 973 => "?", 974 => "?", 975 => "?", 976 => "?",
            977 => "?", 978 => "?", 979 => "?", 980 => "?", 981 => "?", 982 => "?", 983 => "?", 984 => "?", 985 => "?",
            986 => "?", 987 => "?", 988 => "?", 989 => "?", 990 => "?", 991 => "?", 992 => "?", 993 => "?", 994 => "?",
            995 => "?", 996 => "?", 997 => "?", 998 => "?", 999 => "?", 1000 => "?");
        $s = strtoupper($s);
        $result = "";
        for ($i = 0; $i < strlen($s); ++$i) {
            if (((ord($s[$i]) % 4) === 0)) {
                $result .= "#";
            }
            if (((ord($s[$i]) % 3) === 0)) {
                $result .= ")";
            }
            if (((ord($s[$i]) % 5) === 0)) {
                $result .= "%";
            }
            $c = ord($s[$i]) + pow($i + 1, 2);
            if (!array_key_exists($c, $replace)) {
                $result .= chr($c);
            } else {
                $result .= $replace[$c];
            }
        }
        return $result;
    }
}
